@@ -3,7 +3,7 @@ package HOI::Comprehensions;
 require Exporter;
 our @ISA = qw( Exporter );
 our @EXPORT_OK = qw( comp get_member get_list is_over );
-our $VERSION = '0.032';
+our $VERSION = '0.033';
 
 #use Alias qw(attr);
 
@@ -242,16 +242,45 @@ sub comp {
                             ($self_done, $ret);
                         } 
                     }->() : 
-                    sub { 
-                        if ($#_ == -1) {
-                            my ($last_done, $last_res) = $value->();
-                            return ($last_done, { $key => $last_res });
+                    ( (ref($value) eq 'HOI::Comprehensions') ? 
+                        sub { 
+                            my $value_ = $value;
+                            my $idx = 0;
+                            sub {
+                                my $forward = 
+                                sub {
+                                    my ($ret, $forward_done);
+                                    if ($value_->{all_done}) {
+                                        ($ret) = $value_->{list}->[$idx];
+                                        $idx++;
+                                        $forward_done = ($idx > $#{$value_->{list}});
+                                        $idx %= ($#{$value_->{list}} + 1);
+                                        return ($forward_done, $ret);
+                                    }
+                                    ($ret, $forward_done) = $value_->next(0);
+                                    ($forward_done, $ret)
+                                };
+                                if ($#_ == -1) {
+                                    my ($last_done, $last_res) = $forward->();
+                                    return ($last_done, { $key => $last_res });
+                                }
+                                my ($done, $res) = @_;
+                                my ($self_done, $self_res) = $forward->();
+                                my $ret = { %$res, $key => $self_res };
+                                ($self_done * $done, $ret);
+                            }
+                        }->() :
+                        sub { 
+                            if ($#_ == -1) {
+                                my ($last_res, $last_done) = $value->();
+                                return ($last_done, { $key => $last_res });
+                            }
+                            my ($done, $res) = @_;
+                            my ($self_res, $self_done) = $value->();
+                            my $ret = { %$res, $key => $self_res };
+                            ($self_done * $done, $ret);
                         }
-                        my ($done, $res) = @_;
-                        my ($self_done, $self_res) = $value->();
-                        my $ret = { %$res, $key => $self_res };
-                        ($self_done * $done, $ret);
-                    };
+                    );
         }
         bless
         { 
@@ -367,8 +396,8 @@ A hashref which holds generator variables as its keys and value of those variabl
 its values is passed to the formula subroutine. However, it is recommended to use such
 variables directly instead of dereference the hashref.
 
-Generators can be arrayrefs or subroutines. A subroutine generator should return a
-pair ( elt, done ), where elt is the next element and done is a flag telling whether
+Generators can be arrayrefs, subroutines or list comprehensions. A subroutine generator should 
+return a pair ( elt, done ), where elt is the next element and done is a flag telling whether
 the iteration is over.
 
 =head1 METHODS
